SystemOrganization addCategory: #'XMLRPC-Client-Core'!

Error subclass: #XMLRPCException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCException commentStamp: 'dls 11/15/2002 03:05' prior: 0!
Exception thrown when unexpected events occur during XML-RPC processing.!

XMLRPCException subclass: #XMLRPCFaultException
	instanceVariableNames: 'faultCode faultMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCFaultException commentStamp: 'dls 11/15/2002 03:05' prior: 0!
Exception thrown to indicate an XML-RPC fault.!

!XMLRPCFaultException class methodsFor: 'signaling' stamp: 'dls 11/15/2002 22:19'!
signal: faultCode withMessage: faultMessage 
	"Signal an XMLRPC fault with the given fault code and message."
	XMLRPCFaultException new faultCode: faultCode ; faultMessage: faultMessage ; signal: faultMessage! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:17'!
faultCode
	"Get the fault code."
	^ faultCode! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:17'!
faultCode: to
	"Set the fault code to the given value."
	faultCode := to! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:18'!
faultMessage
	"Get the fault message."
	^ faultMessage! !

!XMLRPCFaultException methodsFor: 'accessing' stamp: 'dls 11/15/2002 22:18'!
faultMessage: to
	"Set the fault message to the given value."
	faultMessage := to! !

Object subclass: #XMLRPCDateTime
	instanceVariableNames: 'date time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCDateTime commentStamp: 'dls 11/15/2002 02:56' prior: 0!
DateTime object for sending and receiving dates via XML-RPC.

	See the convenience class methods for easily creating instances of this class.!

!XMLRPCDateTime class methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:23'!
fromDate: d time: t 
	"Get a new XMLRPCDateTime instance from the given date and time."
	^ XMLRPCDateTime new date: d;
		 time: t! !

!XMLRPCDateTime class methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:30'!
fromISO8601String: s
	"Get a new XMLRPCDateTime instance from the given ISO8601 string."
	^ XMLRPCDateTime new decodeISO8601: s! !

!XMLRPCDateTime class methodsFor: 'instance creation' stamp: 'dls 11/15/2002 02:31'!
now 
	"Get a new XMLRPCDateTime instance representing the current time."
	^ XMLRPCDateTime new date: (Date today); time: (Time now)! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'dls 11/18/2002 22:55'!
= anObject
	"True if the other object is an XMLRPCDateTime representing the same date and time."

	| rv |
	rv := false.

	((self class) == (anObject class))
		ifTrue: [ rv := ( (self date) = (anObject date) ) & ((self time) = (anObject time)) ].

	^ rv ! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:00'!
date
	"Get the date portion of the datetime."
	^ date! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:01'!
date: d
	"Set the date portion of the datetime."
	date := d! !

!XMLRPCDateTime methodsFor: 'xmlrpc' stamp: 'dls 11/15/2002 02:28'!
decodeISO8601: s 
	"Set the date and time in this object by parsing the ISO8601 string."
	| d t Y M D |
	Y := (s copyFrom: 1 to: 4) asInteger.
	M := (s copyFrom: 5 to: 6) asInteger.
	D := (s copyFrom: 7 to: 8) asInteger.
	d := Date
				newDay: D
				month: M
				year: Y.
	t := Time readFrom: ((s copyFrom: 10 to: 17) readStream).
	self date: d.
	self time: t! !

!XMLRPCDateTime methodsFor: 'xmlrpc' stamp: 'dls 11/15/2002 02:19'!
encodeISO8601
	| rv |
	"Encode me into an ISO8601 datetime for XML-RPC"
	rv := Text new.

	rv append: (date year asString).
	rv append: (self pad: (date monthIndex asString) to: 2 with: '0').
	rv append: (self pad: (date dayOfMonth asString) to: 2 with: '0').
	rv append: 'T'.
	rv append: (time print24).

	^ rv asString! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'dls 11/18/2002 23:01'!
hash
	"Predictable hash for this object."
	^ date hash bitXor: (time hash)! !

!XMLRPCDateTime methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:20'!
pad: in to: preferredSize with: thing 
	"Return a string with in padded on the left with the given number of  
	the given thing."
	| rv |
	rv := Text new.
	rv append: in asString.
	
	[ rv size < preferredSize ] whileTrue: [
		rv prepend: thing
	].

	^ rv asString! !

!XMLRPCDateTime methodsFor: 'printing' stamp: 'dls 11/15/2002 02:55'!
printOn: aStream
	"Send it to the stream."
	aStream nextPutAll: (self encodeISO8601).! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'dls 11/15/2002 02:01'!
time
	"Get the time portion of the datetime."
	^ time! !

!XMLRPCDateTime methodsFor: 'accessing' stamp: 'SantiagoBragagnolo 9/19/2012 15:21'!
time: t
	"set the time portion of the datetime."
	time := Time fromSeconds: t seconds truncated ! !

Object subclass: #XMLRPCDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCDecoder commentStamp: 'dls 11/15/2002 02:58' prior: 0!
XML-RPC decoder.

This class will take XML-RPC results and translate them into smalltalk types.  The following translations will take place:

	boolean  -> True or False
	string    -> String
	double   -> Float class
	i4         -> SmallInteger
	struct    -> Dictionary
	array    -> Array
	dateTime.iso8601 -> XMLRPCDateTime
	base64   -> Stream!

!XMLRPCDecoder methodsFor: 'decoding' stamp: 'SantiagoBragagnolo 9/14/2012 12:22'!
decode: xml 
	"Decode an XML Element representing an XML-RPC response."
	"xml inspect."
	| el0 |
	
	xml elements isEmpty ifTrue: [^ xml contentString].
	
	"Figure out what to do based on the name of the element (el0 is usually  a required argument."
	
	el0 := xml elements at: 1.
	(el0 name) caseOf: {
		[ #array ]					->	[ ^ self decodeArray: el0 ].
		[ #struct ]					-> 	[ ^ self decodeStructFromDictionary: el0 ].
		[ #param ] 					-> 	[ ^ self decode: el0 ].
		[ #value ] 					->	[ ^ self decode: el0 ].
		[ #string ] 					->	[ ^ el0 contentString ].
		[ #i4 ] 						->	[ ^ SmallInteger readFrom: el0 contentString readStream ].
		[ #int ] 						->	[ ^ SmallInteger readFrom: el0 contentString readStream ].
		[ #double ] 					->	[ ^ Float readFrom: el0 contentString readStream ].
		[ #boolean ]				->	[ ^ el0 contentString = '1' ].
		[ #'dateTime.iso8601' ]		->	[ ^ XMLRPCDateTime fromISO8601String: el0 contentString ].
		[ #base64 ] 				->	[ ^ Base64MimeConverter mimeDecodeToBytes: el0 contentString readStream ].
	} otherwise: [ 
		XMLRPCException signal: 'Unhandled type!!  ' , xml name. 
	].! !

!XMLRPCDecoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:04'!
decodeArray: xml 
	"Decode the Array."
	| rv |
	rv := OrderedCollection new.
	(xml elements at: 1) elements do: [ 
		:el | 
		rv add: (self decode: el)
	].

	^ rv asArray! !

!XMLRPCDecoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:06'!
decodeStruct: xml
	"Decode the Struct."
	| rv |
	rv := Dictionary new.

	xml elements  do: [ 
		:el | 
		el nodes do: [ 
			:eachNode | 
			| name value | 
			name := ( eachNode elementAt: 'name') contentString .
			value := ( self decode: (eachNode elementAt: 'value') ).
			rv at: name put: value.
		].
	].	
	^ rv

! !

!XMLRPCDecoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:10'!
decodeStructFromDictionary: xml
	"Decode the Struct."

	| rv name value |
	name := nil.
	value := nil.
	rv := Dictionary new.
	
	xml elements do: [ 
		:el | 
		el nodes do: [ 
			:eachNode | 
			eachNode name = 'name' ifTrue: [
				name:= eachNode contentString 
			] ifFalse: [
				value := self decode: eachNode.
				rv at: name put: value.
				name := value := nil.
			]
		]
	].	

	^ rv

! !

Object subclass: #XMLRPCEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCEncoder commentStamp: 'dls 11/15/2002 02:58' prior: 0!
Encode objects into proper XML-RPC types.

The following types are understood (smalltalk types in parens):

	Booleans (True and False)
	Strings (String)
	Doubles (Float class)
	Integers (SmallInteger class)
	Struct (Dictionary)
	Array (Array)
	dateTime.iso8601 (XMLRPCDateTime)
	base64 (Stream)!

!XMLRPCEncoder methodsFor: 'encoding' stamp: 'SantiagoBragagnolo 9/14/2012 12:17'!
encode: val 
	"Encode the given value for inclusion in an XML-RPC request or  
	response.  
	 
	Returns the encoded value.  
	"
	val caseOf: {
		"Booleans"
			[ True ] 			->	[ ^ self wrapVal: '1' type: 'boolean' ].
			[ true ] 				->	[ ^ self wrapVal: '1' type: 'boolean' ].
			[ False ] 			-> 	[ ^ self wrapVal: '0' type: 'boolean' ].
			[ false ] 			-> 	[ ^ self wrapVal: '0' type: 'boolean' ].
	} otherwise: [
		(val class) caseOf: {
			"Numbers"
			[ SmallInteger ]		->	[ ^ self wrapVal: val asString type: 'i4' ].
			[ Float ]	 			->	[ ^ self wrapVal: val asString type: 'double' ].
			
			"Strings"		
			[ String ] 			->	[ ^ self wrapVal: (self normalize: val) ].
			[ ByteString ] 		-> 	[ ^ self wrapVal: (self normalize: val) ].
			
			"Date"
			[ XMLRPCDateTime] -> 	[ ^ self wrapVal: val encodeISO8601 type: 'dateTime.iso8601' ].

			"Collections"
			[ Array ]			-> 	[ ^ self wrapVal: (self encodeArray: val) type: 'array' ].
			[ Dictionary ] 		-> 	[ ^ self 	wrapVal: (self encodeStruct: val) type: 'struct'].
					
		} otherwise: [
			val isStream ifTrue: [
				^ self wrapVal: (Base64MimeConverter mimeEncode: val) contents type: 'base64'
			] ifFalse: [
				"Unhandled type."
				XMLRPCException signal: 'Unhandled type:  ' , val className
			].
		].
	].
	








! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:17'!
encodeArray: a 
	"Encode an array."
	| rv |
	rv := Text new.
	rv append: '<data>'.
	a do: [ :el | 
		rv append: (self encode: el)
	].
	rv append: '</data>'.
	^ rv asString! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:18'!
encodeStruct: s 
	"Encode a struct."
	| rv |
	rv := Text new.
	rv append: '<struct>'.

	s keysAndValuesDo: [
		:key :val | 
		rv append: '<member><name>' , (self normalize: key) , '</name>', (self encode: val) , '</member>'.
	].

	rv append: '</struct>'.
				
	^ rv asString! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/14/2012 12:19'!
normalize: s 
	"Normalize an XML string."
	| rv map |

	map := Dictionary new.
	map at: $< put: '&lt;'.
	map at: $> put: '&gt;'.
	map at: $& put: '&amp;'.
	
	rv := Text new.
	
	s do: [:c | 
		rv append: (map at: c ifAbsent: [c]).
	].
	^ rv asString.! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/11/2002 21:16'!
wrapVal: val
	"Wrap the given value in <value> tags, assuming it's already a string."
	^ self wrapVal: val type: 'string'! !

!XMLRPCEncoder methodsFor: 'private' stamp: 'dls 11/11/2002 23:02'!
wrapVal: val type: t 
	"Wrap the given value in <value> and <t>"
	^ '<value><' , t , '>' , val , '</' , t , '></value>'! !

Object subclass: #XMLRPCProxy
	uses: TLogger
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Client-Core'!

!XMLRPCProxy commentStamp: '<historical>' prior: 0!
Proxy class to issue XML-RPC requests.

	Example (invoking a method that takes a single integer argument):

	u := Url absoluteFromText: 'http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC'.
	proxy := XMLRPCProxy withUrl: u.
	proxy invokeMethod: 'echo.echoInt' withArgs: #(1).
	
	Flickr Example:
	
	| url proxy r |

	url := Url absoluteFromText: 'http://api.flickr.com/services/xmlrpc/'.

	proxy := XMLRPCProxy new url: url.

	d := Dictionary new.
	d at: 'name' put: 'flickruser'.
	d at: 'api_key' put: 'flickrkey'.

	r := proxy invokeMethod: 'flickr.test.echo' withStruct: d.



!

XMLRPCProxy class
	uses: TLogger classTrait
	instanceVariableNames: ''!

!XMLRPCProxy class methodsFor: 'instance creation' stamp: 'GermanArduino 11/30/2010 15:04'!
withUrl: u
	"Get a new instance of XMLRPCProxy to connect to a given URL."
	^ self new url: u! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 11/30/2010 14:41'!
buildParamsFrom: anArrayOfArguments

	| params encoder |
	encoder := self encoderClass new.
	params := String new writeStream.
	anArrayOfArguments do: [ :p | 
		params nextPutAll: '<param>', (encoder encode: p), '</param>' ].
	^ params contents.! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 11/30/2010 14:45'!
buildXmlRpc: methodString withArgs: anArray 

	"Build the XMLRPC data required to invoke the given method with the 
	given collection of arguments."

	^ '<?xml version="1.0"?><methodCall><methodName>' , methodString, '</methodName><params>' , (self buildParamsFrom: anArray), '</params></methodCall>'.! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 11/28/2010 12:10'!
buildXmlRpc: method withStruct: args 
	"Build the XMLRPC data required to invoke the given method with the 
	given collection of arguments."
	| params e rv |
	e := XMLRPCEncoder new.
	params := Text new.

	params append: '<param><value>'	, (e encodeStruct: args) , '</value></param>'.
	
	rv := '<?xml version="1.0"?><methodCall><methodName>' , method , '</methodName><params>' , params , '</params></methodCall>'.
	
	^ rv! !

!XMLRPCProxy methodsFor: 'private' stamp: 'GermanArduino 11/30/2010 14:47'!
encoderClass

	^ XMLRPCEncoder.! !

!XMLRPCProxy methodsFor: 'invoking' stamp: 'GermanArduino 11/30/2010 14:52'!
invokeMethod: aString 
	"Invoke a method on the XML-RPC server with no arguments."
	
	^ self invokeMethod: aString withArgs: #()! !

!XMLRPCProxy methodsFor: 'invoking' stamp: 'SantiagoBragagnolo 9/19/2012 12:22'!
invokeMethod: aString withArgs: anArray 
	"Invoke a method on the XML-RPC server.

	args is a list of parameters to be passed.  For example, if the remote method is called ``test.Method'' and takes two ints, you'd invoke is like this:

	anXMLRPCProxy invokeMethod: 'test.method' withArgs: #(19 77)
"
	| request response |
	
	self debug: ['invokeMethod:', aString , 'withArgs:', anArray asString].
	
	request := self buildXmlRpc: aString withArgs: anArray.
	response := self sendXmlRpc: request.
	^ self processResponse: response.! !

!XMLRPCProxy methodsFor: 'invoking' stamp: 'GermanArduino 4/1/2011 20:06'!
invokeMethod: method withStruct: args
	"Invoke a method on the XML-RPC server.

	A value can also be of type <struct>.

	A <struct> contains <member>s and each <member> contains a <name> and a <value>.

	We call a struct using a Dictionay, because each entry already have 2 elements."

	| req res rv |


	" TODO A dictionary is requested here, check to document and improve method name, modify test"
	
		args class = Dictionary 
		ifFalse: [ XMLRPCFaultException signal: 100 withMessage: 'You must pass a Dictionary to call a struct' ]
		ifTrue: [ 
			req := self buildXmlRpc: method withStruct: args.
			res := self sendXmlRpc: req.
			rv := self processResponse: res.
			^ rv ]	
		
		
		
		
! !

!XMLRPCProxy methodsFor: 'private' stamp: 'dls 11/15/2002 00:23'!
processFault: xmlFault 
	"Process an XML-RPC fault into an XMLRPCFaultException"
	| d decoder |
	decoder := XMLRPCDecoder new.
	d := decoder decode: xmlFault.
	XMLRPCFaultException
		signal: (d at: 'faultCode')
		withMessage: (d at: 'faultString').! !

!XMLRPCProxy methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/19/2012 14:55'!
processResponse: xmlStream 
	"Process the results."
	| xmldoc methodResponse fault parameters content  |
	
	xmldoc := XMLDOMParser parseDocumentFrom: xmlStream.
	
	methodResponse := xmldoc elementAt: 'methodResponse'. 
	
	methodResponse	ifNil: [ 
		XMLRPCException signal: 'methodResponse not found in XML Stream.' 
	].
	
	"See if it's a fault response."
	
	fault := methodResponse elementAt: 'fault'.
	fault ifNotNil: [ 
		self processFault: fault
	].
	
	"If we made it this far, it's a valid response, look for the results."
	parameters := methodResponse elementAt: 'params'.
	parameters ifNil: [
		XMLRPCException signal: 'Params not found in XML-RPC response.'
	].

	"Decode, and return the response."

	^ XMLRPCDecoder new decode: parameters
	

! !

!XMLRPCProxy methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/19/2012 15:28'!
sendXmlRpc: data 
	"Send the XML-RPC data and get the response. Return the raw XML."
	| s req res xml |
	
	res := ZnEasy post: url data: (ZnStringEntity xml: data).
	
	"Make sure the result was valid."
	(res isSuccess and: [ res contentType sub = 'xml' ]) ifFalse: [ self error: ' ', res statusLine asString ].
	
	
	^ res contents.
	
	! !

!XMLRPCProxy methodsFor: 'accessing' stamp: 'dls 11/14/2002 23:18'!
url
	"Get the URL against which requests will be invoked."
	^ url! !

!XMLRPCProxy methodsFor: 'accessing' stamp: 'dls 11/14/2002 23:17'!
url: aUrl
	"Set the URL that will be servicing the requests."
	url := aUrl! !
